#!/usr/bin/env python

"""
Copyright 2014 VMware, Inc.  All rights reserved. -- VMware Confidential
"""

from VmodlDecorators import \
    ManagedType, DataType, EnumType, \
    Attribute, Method, Param, Return, \
    F_LINK, F_LINKABLE, F_OPTIONAL, \
    VmodlDecoratorException, RegisterVmodlTypes
from pyVmomi import Vmodl

## Vmodl names
#
_VERSION = "vim.version.version10"


class VsanObjectIdentity:
   _name = 'vim.cluster.VsanObjectIdentity'

   @DataType(name=_name, version=_VERSION)
   def __init__(self):
      pass

   """
   VSAN UUID of the object
   """
   @Attribute(parent=_name, typ="string")
   def uuid(self): pass

   """
   Can be one of: 'vmswap', 'vdisk', 'namespace', 'vmem', 'stats', 'other'
   """
   @Attribute(parent=_name, typ="string")
   def type(self): pass

   """
   Instance UUID of the VM this object belongs to
   """
   @Attribute(parent=_name, typ="string", flags=F_OPTIONAL)
   def vmInstanceUuid(self): pass

   """
   Reference to VM this object belongs to. If the API was executed against a host
   (even if proxied through VC) the VM reference will use the MO Ref of the host, 
   not that of VC. The vmInstanceUuid should be used instead to match to the VC MO Ref
   in such cases.
   """
   @Attribute(parent=_name, typ="vim.VirtualMachine", flags=F_OPTIONAL)
   def vm(self): pass

   """
   Description
   """
   @Attribute(parent=_name, typ="string", flags=F_OPTIONAL)
   def description(self): pass


class VsanObjSystem:
   _name = 'vim.host.VsanObjSystem'

   @ManagedType(name=_name, version=_VERSION)
   def __init__(self):
      pass

   """
   Map from VSAN object UUIDs to their identities.
   
   The results contain the identities of all objects that could be resolved. The 
   identity includes things like the object type, references to the VM the object 
   belongs to and such. If called against a host, the cluster argument is ignored
   and only those objects are resolved which can be resolved by that host. For VM related
   objects a host can only resolve the objects if the VM they belong to is registered
   on that host. If called against vCenter, all connected hosts will be contacted for
   this information and the results will be aggregated. Any VMs registered with hosts
   which are disconnected will hence not be resolved.
   
   @param cluster VSAN cluster. Ignored if called against host.
   @param objUuids List of VSAN object UUIDs for which to fetch identities.
   @return Object information structure
   """
   @Method(parent=_name, wsdlName = "VsanQueryObjectIdentities",
           faults=["vmodl.fault.NotSupported"])
   @Param(name="cluster", typ="vim.ComputeResource")
   @Param(name="objUuids", typ="string[]")
   @Return(typ="vim.cluster.VsanObjectIdentity[]", flags=F_OPTIONAL)
   def QueryObjectIdentities(self, cluster, objUuids):
      pass

# Register managed object types
RegisterVmodlTypes()
