#!/usr/bin/env python

"""
Copyright 2014 VMware, Inc.  All rights reserved. -- VMware Confidential

"""
__author__ = "VMware, Inc"

from pyVmomi import vim, vmodl
from MoManager import GetMoManager
import json
import pyVmomi
import re
import logging
import six

def CollectMultiple(content, objects, parameters, handleNotFound=True):
   if len(objects) == 0:
      return {}
   result = None
   # XXX: Does this cause an RPC?
   pc = content.propertyCollector
   propSet = [vim.PropertySpec(
      type=objects[0].__class__,
      pathSet=parameters
   )]

   while result == None and len(objects) > 0:
      try:

         objectSet = []
         for obj in objects:
            objectSet.append(vim.ObjectSpec(obj=obj))
         specSet = [vim.PropertyFilterSpec(objectSet=objectSet, propSet=propSet)]
         result = pc.RetrieveProperties(specSet=specSet)
      except vim.ManagedObjectNotFound as ex:
         objects.remove(ex.obj)
         result = None

   out = {}
   for x in result:
      out[x.obj] = {}
      for y in x.propSet:
         out[x.obj][y.name] = y.val
   return out

def ConnectToLocalHostd(profiler = None):
   version = "vim.version.version10"
   port = 443
   conn = pyVmomi.SoapStubAdapter('localhost', port,
                                  version=version, path='/sdk')
   si = vim.ServiceInstance("ServiceInstance", conn)
   content = si.RetrieveServiceContent()
   sm = content.sessionManager
   localTicket = sm.AcquireLocalTicket('root')
   pw = open(localTicket.passwordFilePath, 'r').read()
   sm.Login(localTicket.userName,pw)
   return (si, content)

class VsanObjSystemImpl(vim.host.VsanObjSystem):
   def QueryObjectIdentities(self, cluster, objUuids):
      if True:
         si, content = ConnectToLocalHostd()
         containerView = content.viewManager.CreateContainerView(
            container = content.rootFolder,
            recursive = True,
            type = [vim.VirtualMachine]
         )

         vms = containerView.view

         containerView.DestroyView()

         vmsProps = CollectMultiple(
            content,
            vms,
            ['name', 'summary.config.instanceUuid',
             'layoutEx.file', 'summary.config.vmPathName']
         )
      out = []
      for vm in vms:
         vmProps = vmsProps[vm]
         files = dict([(f.backingObjectId, f)
                       for f in vmProps['layoutEx.file']
                       if f.backingObjectId is not None and f.backingObjectId in objUuids])
         vmProps.setdefault('summary.config.instanceUuid', None)
         if len(files) > 0:
            for uuid, file in six.iteritems(files):
               typeMap = {
                  'diskDescriptor': 'vdisk',
                  'swap': 'vmswap',
                  'snapshotMemory': 'vmem',
                  'suspendMemory': 'vmem',

               }
               typ = 'other'
               if file.type in typeMap:
                  typ = typeMap[file.type]
               out.append(vim.cluster.VsanObjectIdentity(
                  uuid = uuid,
                  type = typ,
                  description = file.name,
                  vmInstanceUuid = vmProps['summary.config.instanceUuid'],
                  vm = vm
               ))
            logging.info("UUIDs %s are part of files of VM %s" %
                         (list(files.keys()), vmProps['name']))
         m = re.match(r"^\[([^\]]*)\] ([^\/]*)\/", vmProps['summary.config.vmPathName'])
         if m:
            nsObjUuid = m.group(2)
            if nsObjUuid in objUuids:
               logging.info("UUID %s is NS of VM %s" % (nsObjUuid, vmProps['name']))
               out.append(vim.cluster.VsanObjectIdentity(
                  uuid = nsObjUuid,
                  type = 'namespace',
                  description = 'VM Home',
                  vmInstanceUuid = vmProps['summary.config.instanceUuid'],
                  vm = vm
               ))

      return out

# Add managed objects during import
GetMoManager().RegisterObjects([VsanObjSystemImpl("ha-vsan-obj-system")])

