#!/usr/bin/python

# **********************************************************
# Copyright 2006 VMware, Inc.  All rights reserved. -- VMware Confidential
# **********************************************************

import getopt
import os
import sys
from pyVmomi import types, Vmodl, Vim, SoapStubAdapter
#from pyVmomi.SoapAdapter import Serialize
from pyVmomi import VmomiSupport
from pyVim import configSerialize

def AddCpuidOption(optionArray, level, regName):
   modeOptionDef = Vim.Option.OptionDef()
   modeOptionDef.key = "cpuidMask.mode." + level + "." + regName;
   modeOptionDef.label = "cpuidMask.mode." + level + "." + regName;
   modeOptionDef.summary = "Controls the use of Config.Defaults.cpuidMask.val." + level + "." + regName + "; legal values \"mask\", \"clobber\", or \"disable\".";

   modeOptionDef.optionType = Vim.Option.StringOption()
   modeOptionDef.optionType.valueIsReadonly = False
   modeOptionDef.optionType.defaultValue = "disable"

   valOptionDef = Vim.Option.OptionDef()
   valOptionDef.key = "cpuidMask.val." + level + "." + regName;
   valOptionDef.label = "cpuidMask.val." + level + "." + regName;
   valOptionDef.summary = "CPUID override that may be applied to level " + level + " register " + regName + "; controlled by Config.Defaults.cpuidMask.mode." + level + "." + regName + ".";

   valOptionDef.optionType = Vim.Option.StringOption()
   valOptionDef.optionType.valueIsReadonly = False
   valOptionDef.optionType.defaultValue = ""

   optionArray.append(modeOptionDef);
   optionArray.append(valOptionDef);

def MakeConfigDefSpec():
   optionDef = Vim.Option.OptionDef()
   optionDef.key = "security.host.ruissl";
   optionDef.label = "security.host.ruissl";
   optionDef.summary = "Require SSL to be used when communicating with the host over port 902.";
   
   optionDef.optionType = Vim.Option.BoolOption()
   optionDef.optionType.valueIsReadonly = False
   optionDef.optionType.defaultValue = True
   optionDef.optionType.supported = True

   configOptionDefs = [optionDef]

   AddCpuidOption(configOptionDefs, "0", "eax");
   AddCpuidOption(configOptionDefs, "0", "ebx");
   AddCpuidOption(configOptionDefs, "0", "ecx");
   AddCpuidOption(configOptionDefs, "0", "edx");
   AddCpuidOption(configOptionDefs, "1", "eax");
   AddCpuidOption(configOptionDefs, "1", "ebx");
   AddCpuidOption(configOptionDefs, "1", "ecx");
   AddCpuidOption(configOptionDefs, "1", "edx");
   AddCpuidOption(configOptionDefs, "80000000", "eax");
   AddCpuidOption(configOptionDefs, "80000000", "ebx");
   AddCpuidOption(configOptionDefs, "80000000", "ecx");
   AddCpuidOption(configOptionDefs, "80000000", "edx");
   AddCpuidOption(configOptionDefs, "80000001", "eax");
   AddCpuidOption(configOptionDefs, "80000001", "ebx");
   AddCpuidOption(configOptionDefs, "80000001", "ecx");
   AddCpuidOption(configOptionDefs, "80000001", "edx");
   AddCpuidOption(configOptionDefs, "80000008", "eax");
   AddCpuidOption(configOptionDefs, "80000008", "ebx");
   AddCpuidOption(configOptionDefs, "80000008", "ecx");
   AddCpuidOption(configOptionDefs, "80000008", "edx");
   AddCpuidOption(configOptionDefs, "8000000A", "eax");
   AddCpuidOption(configOptionDefs, "8000000A", "ebx");
   AddCpuidOption(configOptionDefs, "8000000A", "ecx");
   AddCpuidOption(configOptionDefs, "8000000A", "edx");

   return Vim.Option.OptionDef.Array(configOptionDefs)

def MakeSettingsDefSpec():
   optionDef = Vim.Option.OptionDef()
   optionDef.key = "guest.commands.sharedPolicyRefCount";
   optionDef.label = "guest.commands.sharedPolicyRefCount";
   optionDef.summary = "Reference count to enable guest operations.";

   optionDef.optionType = Vim.Option.IntOption()
   optionDef.optionType.valueIsReadonly = False
   optionDef.optionType.defaultValue = 0
   optionDef.optionType.min = 0
   optionDef.optionType.max = 0x7fffffff

   return Vim.Option.OptionDef.Array([optionDef])

def main():
   print "-------------------- Config Definitions --------------------"
   configDef = MakeConfigDefSpec()
   print configSerialize.SerializeToConfig(configDef)
   print "------------------------------------------------------------"

   print "-------------------- Setting Definitions --------------------"
   settingsDef = MakeSettingsDefSpec()
   print configSerialize.SerializeToConfig(settingsDef)
   print "------------------------------------------------------------"

# Start program
if __name__ == "__main__":
   main()

