#!/usr/bin/python

from __future__ import print_function

import sys
from pyVmomi import Vim, Hostd
from pyVmomi import VmomiSupport
from pyVim.connect import Connect, Disconnect
from pyVim.task import WaitForTask
from pyVim import host
from pyVim import arguments
import atexit

def GetSystemDebugManager(hostIn, userIn, pwdIn):
   # Connect
   si = Connect(host=hostIn,
                user=userIn,
                pwd=pwdIn,
                version="vim.version.version9")
   atexit.register(Disconnect, si)

   hostSystem = host.GetHostSystem(si)
   icMgr = hostSystem.RetrieveInternalConfigManager()
   return icMgr.GetSystemDebugManager()

def main():
   supportedArgs = [ (["h:", "host="], "localhost", "Host name", "host"),
                     (["u:", "user="], "root", "User name", "user"),
                     (["p:", "pwd="], "", "Password", "pwd")]

   supportedToggles = [ (["usage", "help"], False, "Show usage information", "usage") ]

   args = arguments.Arguments(sys.argv, supportedArgs, supportedToggles)
   if args.GetKeyValue("usage") == True:
      args.Usage()
      sys.exit(0)

   sdMgr = GetSystemDebugManager(args.GetKeyValue("host"),
                args.GetKeyValue("user"),
                args.GetKeyValue("pwd"))
   if sdMgr == None:
      print("system debug manger instance is NULL!")
      sys.exit(0)

   #get supported stats definition
   procInfo = sdMgr.QueryProcessInfo()
   print(procInfo)

# Start program
if __name__ == "__main__":
    main()
