#!/usr/bin/python

from __future__ import print_function

import sys
import time
import thread
from pyVmomi import Vim, SoapStubAdapter
import connect
import atexit


def GetRootFolder(si):
    content = si.RetrieveContent()
    rootFolder = content.GetRootFolder()
    return rootFolder

def Indent(depth):
    return "   " * depth

def WalkFolder(folder, depth=0):
    indent = Indent(depth)
    childEntities = folder.GetChildEntity()
    print(indent + "%s: %s" % (folder.GetName(), folder))

    for i in range(len(childEntities)):
        childEntity = childEntities[i]
        WalkManagedEntity(childEntity, depth + 1)

def WalkManagedEntity(entity, depth):
    if isinstance(entity,Vim.Folder):
        WalkFolder(entity, depth)
    elif isinstance(entity, Vim.Datacenter):
        WalkDatacenter(entity, depth)
    elif isinstance(entity, Vim.VirtualMachine):
        WalkVirtualMachine(entity, depth)
    elif isinstance(entity, Vim.HostSystem):
        WalkHostSystem(entity, depth)
    elif isinstance(entity, Vim.ComputeResource):
        WalkComputeResource(entity, depth)
    elif isinstance(entity, Vim.ClusterComputeResource):
        WalkComputeResource(entity, depth)
    elif isinstance(entity, Vim.Datastore):
	WalkDatastore(entity, depth)
    elif isinstance(entity, Vim.Network):
        WalkNetwork(entity, depth)
    else:
        print(Indent(depth) + entity)

def PrintManagedEntity(entity, depth):
    if isinstance(entity,Vim.Folder):
        print(" %s: %s" % (entity.GetName(), entity))
    elif isinstance(entity, Vim.Datacenter):
        print(" %s: %s" % (entity.GetName(), entity))
    elif isinstance(entity, Vim.VirtualMachine):
        print(" %s: %s" % (entity.GetSummary().GetConfig().GetName(), entity))
    elif isinstance(entity, Vim.HostSystem):
        print(" %s: %s" % (entity.GetName(), entity))
    elif isinstance(entity, Vim.ComputeResource):
        print(" %s: %s" % (entity.GetName(), entity))
    elif isinstance(entity, Vim.ClusterComputeResource):
        print(" %s: %s" % (entity.GetName(), entity))
    elif isinstance(entity, Vim.Datastore):
	print(" Datastore: %s" % (entity.GetSummary().GetName()))
    elif isinstance(entity, Vim.Network):
	print(" Network: %s" % (entity.GetSummary().GetName()))
    else:
        print(" %s: %s" % (entity.GetName(), entity))

def WalkDatacenter(datacenter, depth=0):
    indent = Indent(depth)

    print(indent + "%s: %s" % (datacenter.GetName(), datacenter))

    hostFolder = datacenter.GetHostFolder()
    vmFolder = datacenter.GetVmFolder()
    dsFolder = datacenter.GetDatastoreFolder()
    ntFolder = datacenter.GetNetworkFolder()
    WalkFolder(hostFolder, depth + 1)
    WalkFolder(vmFolder, depth + 1)
    WalkFolder(dsFolder, depth + 1)
    WalkFolder(ntFolder, depth + 1)
#    datastores = datacenter.GetDatastore()
#    WalkDatastores(datastores, depth + 1)
#    networks = datacenter.GetNetwork()
#    WalkNetworks(networks, depth + 1)

def WalkComputeResource(computeResource, depth=0):
    indent = Indent(depth)

    print(indent + "%s: %s" % (computeResource.GetName(), computeResource))

    resourcePool = computeResource.GetResourcePool()
    WalkResourcePool(resourcePool, depth + 1)

    hostSystems = computeResource.GetHost()
    WalkHostSystems(hostSystems, depth + 1)

    datastores = computeResource.GetDatastore()
    WalkDatastores(datastores, depth + 1)

    networks = computeResource.GetNetwork()
    WalkNetworks(networks, depth + 1)

def WalkResourcePool(resourcePool, depth=0):
    indent = Indent(depth)

    print(indent + "%s: %s" % (resourcePool.GetName(), resourcePool))

def WalkVirtualMachine(vm, depth=0):
    indent = Indent(depth)

    print(indent + "%s: %s" % (vm.GetSummary().GetConfig().GetName(), vm))

def WalkHostSystems(hostSystems, depth=0):
    indent = Indent(depth)
    print(indent + "Hosts:")
    for hostSystem in hostSystems:
        WalkHostSystem(hostSystem, depth + 1)

def WalkHostSystem(hostSystem, depth=0):
    indent = Indent(depth)

    print(indent + "%s: %s" % (hostSystem.GetSummary().GetConfig().GetName(),
                               hostSystem))
    datastores = hostSystem.GetDatastore()
    WalkDatastores(datastores, depth + 1)
    networks = hostSystem.GetNetwork()
    WalkNetworks(networks, depth + 1)

def WalkDatastores(datastores, depth=0):
    indent = Indent(depth)
    print(indent + "Datastores:")
    for datastore in datastores:
        WalkDatastore(datastore, depth + 1)

def WalkDatastore(datastore, depth=0):
    indent = Indent(depth)
    print(indent + datastore.GetSummary().GetName())

def WalkNetworks(networks, depth=0):
    indent = Indent(depth)
    print(indent + "Networks:")
    for network in networks:
        WalkNetwork(network, depth + 1)

def WalkNetwork(network, depth=0):
    indent = Indent(depth)
    print(indent + network.GetSummary().GetName())

def WalkInventory(si):
    print("ServiceInstance Inventory:")
    rootFolder = GetRootFolder(si)
    WalkFolder(rootFolder)

def WalkViewManager(si):
    sic = si.RetrieveContent()

    vm = sic.GetViewManager()
    cAllView = vm.CreateContainerView(sic.GetRootFolder(), None, True)
    print("MangerView:")
    viewlist = cAllView.GetView()
    for view in viewlist:
	    PrintManagedEntity(view, 0)

def main():
   connect.Connect(host='localhost', user='administrator')


   # Create a container view on the whole inventory
   si = connect.GetSi()
   atexit.register(connect.Disconnect, si)

   WalkInventory(si)
   WalkViewManager(si)

# Start program
if __name__ == "__main__":
    main()
